{
=============================================================================
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  |
| AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    |
| IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   |
| ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR  |
| ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL       |
| DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR   |
| SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER   |
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT           |
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY    |
| OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  |
| DAMAGE.                                                                      |
=============================================================================
   Author:  D.J.C. Blijker.
   Date:    22-4-2003
   version: 2.0
   e-mail:  c.blijker@wxs.nl
=============================================================================
  This unit will work in Delphi as well as in Kylix!!!
=============================================================================
  Usage:
    Just instantiate an instance of a THxLogClient
    and use the following procedures to log anything you want:

    procedure LogStr( sStr: string );
    procedure LogFormated( sFormat: string; const Args: array of const );
    procedure LogWarning( iWarningLevel: integer; sStr: string );
    procedure LogWarningFormated( iWarningLevel: integer; sFormat: string;
                                  const Args: array of const );

    Warnings a signaled differently on the logDaemon-side.

    To gain more control over the logging you can call the following:

    procedure UseLogFileName( sName: string );
      Force the use of a specail logfilename..
    procedure UseNrOfLogs( iNrOfLogs: integer );
      Default the logdaemon will use 10 logfiles on a round-robbin basis.
      But you can in/de-crease that at will..
    procedure SetLogSwitchTime( iTimeInMin: integer ); // 0 ==> Don't switch..
      Set the time before the logdaemon will switch to a new log-file
    procedure StartNewLogFile;
      Force the use of a new log-file..
=============================================================================
 By default, connections to the logdaemon are made using the IPC-protocol.
 But you can, if you wish, use the udp-protocol.
 This can be usefull if you don't want to use the serviceseeker and still be
 able to automaticaly locate the logdaemon on the local network segment
=============================================================================
}

{ $ DEFINE USE_UDP_CLIENT}

{$UNDEF USE_UDP_CLIENT}
{$DEFINE USE_SERVICE_SEEKER}

unit LogClient_LogicLayer;

interface

uses
  Windows, SysUtils, IniFiles, Forms, extctrls, Dialogs, Classes,
{$IFDEF USE_SERVICE_SEEKER}
  ServiceSeekerLayer, ServiceBroker_Shared,
{$ENDIF}
  LogDaemon_Common, HxDataExchange, HxTcpIpcUdp;

type
  TNotifyProc = procedure( Sender: TObject )of object;
  TOnError = procedure( Sender: TObject; sMess: string )of object;

  TConnectionType = ( ctIPC, ctUDP );

  // Buffer record
  TBufferRec = record
    iSize: integer;
    pData: pchar;
  end;
  PBufferRec = ^TBufferRec;

  // List of send buffer records
  TBufferList = class( TList )
  protected
    procedure Put( index: integer; pElement: PBufferRec );
    function  Get( index: integer ): PBufferRec;
    procedure Notify( ptr: Pointer; Action: TListNotification ); override;
  public
    property items[ index: integer]: PBufferRec read Get
                                                write Put; default;
    function Add( ptr: PBufferRec ): integer;
  end;

  THxLogClient = class
  private
    sName: string;
    FOnConnection: TNotifyProc;
    FOnConnectionClosed: TNotifyProc;
    FOnError: TOnError;
{$IFNDEF USE_UDP_CLIENT}
    ipcClient: THxIpcClient;
  {$IFNDEF USE_SERVICE_SEEKER}
    sDaemonAddress: string;
  {$ENDIF}
{$ENDIF}
{$IFDEF USE_SERVICE_SEEKER}
    ss: THxServiceSeeker;
{$ENDIF}
{$IFDEF USE_UDP_CLIENT}
    udpClient: THxUdp;
    bLogDaemonConnected: boolean;
    bForcedSend: boolean;
    bUdpConnectionEstabished: boolean;
    bUdpConnected: boolean;
{$ENDIF}
    eConnectionType: TConnectionType;
    bDoLog: boolean;
    lstBuffer: TBufferList;
    sKeepNamedLogName: string;
    iMaxBufferLines: integer;
    tmr: TTimer;
    deDataEx: THxDataExchange;
    function  GetActive: boolean;
    procedure SetActive(const Value: boolean);
    procedure Send( ptr: pointer; iSize: integer );
    procedure DoCreate( sServerAddress: string = LOGDAEMON_ADDRESS );
    procedure Notify( sMess: string );
{$IFNDEF USE_UDP_CLIENT}
    procedure HandleConnecionClosed;
    procedure SendByIpc(ptr: pointer; iSize: integer);
    procedure HandleConnection;
  {$IFNDEF USE_SERVICE_SEEKER}
    procedure HandleIpcTimer( Sender: TObject );
  {$ENDIF}
{$ENDIF}
{$IFDEF USE_UDP_CLIENT}
    procedure SendByUdp( ptr: pointer; iSize: integer );
    procedure HandleUdpData( pBuf: pchar; iSize: integer;
                             sRemoteIP, sPort: string );
    procedure HandleNoUdpServer;
    procedure HandleUdpTimer( Sender: TObject );
    procedure HandleAcknowledge( sIP: string );
    procedure SendGoodBye;
    procedure SendAlive;
{$ENDIF}
    procedure EmergencyDump;
    procedure SetMaxBufferLines( iValue: integer );
    procedure SendBuffer;
{$IFDEF USE_SERVICE_SEEKER}
    procedure HandleServiceAnswer( sServiceName, sAddress, sPort: string;
                                   eProtocol: THxProtocol );
{$ENDIF}
    procedure SendIntroduction; virtual;
    procedure SendIntMessage( iId, iVal: integer );
    procedure SendIntStrMessage( iId, iVal: integer; s1: string );
    procedure SendStrMessage( iId: integer; s1: string );
  public
{$IFNDEF USE_UDP_CLIENT}
  {$IFDEF USE_SERVICE_SEEKER}
    constructor Create;
  {$ELSE}
    constructor Create( sServerAddress: string = LOGDAEMON_ADDRESS );
  {$ENDIF}
{$ENDIF}
{$IFDEF USE_UDP_CLIENT}
    constructor Create;
{$ENDIF}
    destructor Destroy; override;
    property  Active: boolean read GetActive write SetActive;
    property  OnError: TOnError read FOnError write FOnError;
// The log-procedures..
    procedure LogStr( sStr: string );
    procedure LogFormated( sFormat: string; const Args: array of const );
    procedure LogWarning( iWarningLevel: integer; sStr: string );
    procedure LogWarningFormated( iWarningLevel: integer; sFormat: string;
                                  const Args: array of const );
// 'Control' procedures..
    procedure UseLogFileName( sName: string );
    procedure UseNrOfLogs( iNrOfLogs: integer );
    procedure SetLogSwitchTime( iTimeInMin: integer ); // 0 ==> Don't switch..
    procedure StartNewLogFile;
// IPC-notifications..
    property OnConnection: TNotifyProc read FOnConnection write FOnConnection;
    property OnConnectionClosed: TNotifyProc read FOnConnectionClosed
             write FOnConnectionClosed;
    function Connected: boolean;
    property MaxBufferLines: integer read iMaxBufferLines
                                     write SetMaxBufferLines;
  end;

implementation

const
  DEFAULT_MAX_BUFFER_LINES = 1000;


// ===== BufferList implementation
function TBufferList.Add( ptr: PBufferRec ): integer;
begin
  result := inherited Add( ptr );
end;

function TBufferList.Get(index: integer): PBufferRec;
begin
  result := PBufferRec( inherited Get( index ));
end;

procedure TBufferList.Notify( ptr: Pointer; Action: TListNotification);
begin
  if Action = lnDeleted then
  begin
    if Assigned( PBufferRec( ptr )^.pData )then
    try
      Dispose( PBufferRec( ptr )^.pData );
    except
    end;
    PBufferRec( ptr )^.pData := nil;
    Dispose( PBufferRec( ptr ));
  end;
end;

procedure TBufferList.Put( index: integer; pElement: PBufferRec );
begin
  inherited Put( index, pElement );
end;

function GetRandomName: string;
var
  s: string;
  i: integer;
begin
  Randomize;
  SetLength( s, 6 );
  for i := 1 to 6 do
    s[i] := char( Random(26)+Ord('a'));
  result := s;
end;

{$IFNDEF USE_UDP_CLIENT}
  {$IFDEF USE_SERVICE_SEEKER}
constructor THxLogClient.Create;
begin
  eConnectionType := ctIPC;

  ss := THxServiceSeeker.Create;
  ss.OnAnswer := HandleServiceAnswer;
  ss.SeekService( LOGDAEMON_SERVICE, pcIpc );

  DoCreate;
end;
  {$ELSE}
constructor THxLogClient.Create( sServerAddress: string = LOGDAEMON_ADDRESS );
begin
  eConnectionType := ctIPC;
  DoCreate( sServerAddress );
end;
  {$ENDIF}
{$ENDIF}

{$IFDEF USE_UDP_CLIENT}
constructor THxLogClient.Create;
begin
  eConnectionType := ctUDP;

  {$IFDEF USE_SERVICE_SEEKER}
  ss := THxServiceSeeker.Create;
  ss.OnAnswer := HandleServiceAnswer;
  ss.SeekService( LOGDAEMON_SERVICE, pcUdp );
  {$ENDIF}

  bLogDaemonConnected := False;
  DoCreate;
end;
{$ENDIF}

// The 'Real' create
procedure THxLogClient.DoCreate( sServerAddress: string = LOGDAEMON_ADDRESS );
begin
//  sName := Application.Exename;
  sName := GetRandomName;
// Create pre-connect buffer
  lstBuffer := TBufferList.Create;
// Init
  sKeepNamedLogName := '';
  iMaxBufferLines := DEFAULT_MAX_BUFFER_LINES;

// This timer can be used to make sure an udp connection (if applicable)
// is setup properly.
// Or to attempt(s) to setup an Ipc-connection. (In case no use of a
// Service-Seeker is beeing made)

  tmr := TTimer.Create( nil );
  tmr.Enabled := False;

  bDoLog := True; // Active by default..
  deDataEx := THxDataExchange.Create;

{$IFNDEF USE_UDP_CLIENT}
  {$IFNDEF USE_SERVICE_SEEKER}
  ipcClient := THxIpcClient.Create;
  with ipcClient do
  begin
    ServerAddress := sServerAddress;
    Port := LOGDAEMON_PORT;
    OnConnectionClosed := HandleConnecionClosed;
  end;
  sDaemonAddress := sServerAddress;
  tmr.Interval := 1000;
  tmr.OnTimer := HandleIpcTimer;
  tmr.Enabled := True;
  {$ENDIF}
{$ENDIF}

{$IFDEF USE_UDP_CLIENT}
  tmr.Interval := 30000; // 30 seconds
  tmr.OnTimer := HandleUdpTimer;
  bForcedSend := False;
  {$IFNDEF USE_SERVICE_SEEKER}
  udpClient := THxUdp.Create;
  with udpClient do
  begin
    OnDataAvailable := HandleUdpData;
    DestinationAddress := GetBroadCastAddress( CURRENT_NETMASK );
    DestinationPort := LOGDAEMON_PORT;
    ListeningPort := ANY_FREE_PORT;
  end;
  SendIntroduction;
  tmr.Enabled := True;
  {$ENDIF}
{$ENDIF}
end;

{$IFDEF USE_SERVICE_SEEKER}
procedure THxLogClient.HandleServiceAnswer( sServiceName, sAddress,
                                            sPort: string;
                                            eProtocol: THxProtocol );
begin
//  Notify( Format('service answer: ServiceName: %s, ServiceAddress: %s, '+
//                 'ServicePort: %s', [sServiceName, sAddress, sPort]));

{$IFNDEF USE_UDP_CLIENT}
  if Assigned( ipcClient )then
    ipcClient.Free;

  ipcClient := THxIpcClient.Create;
  with ipcClient do
  begin
    ServerAddress := sAddress;
    Port := sPort;
    OnConnectionClosed := HandleConnecionClosed;
    if Connect then
      HandleConnection
    else
      ss.SeekService( LOGDAEMON_SERVICE, pcIpc );
  end;
{$ENDIF}

{$IFDEF USE_UDP_CLIENT}
  if not( Assigned( udpClient ))then
    udpClient := THxUdp.Create;

  with udpClient do
  begin
    OnDataAvailable := HandleUdpData;
    DestinationAddress := sAddress;
    DestinationPort := sPort;
    ListeningPort := ANY_FREE_PORT;
  end;

  SendIntroduction;
  tmr.Enabled := True;

{$ENDIF}
end;
{$ENDIF}

procedure THxLogClient.SetMaxBufferLines( iValue: integer );
begin
  // Copy value
  iMaxBufferLines := iValue;

  // Already more in buffer: delete oldest entries
  while lstBuffer.Count > iMaxBufferLines do
    lstBuffer.Delete( 0 );
end;


// Dump all logs in memory to file as an emergency handling
procedure THxLogClient.EmergencyDump;
var
  sLogName: string;
  f: TextFile;
  i: integer;
  iType: integer;
  iWarnLevel: integer;
  sMessage: string;
  pMyData: pchar;
begin
  // Construct an emergency logfile name
  if sKeepNamedLogName = '' then
    sLogName := Copy( Application.ExeName,
                      1,
                      Length( Application.ExeName ) - 4 )
  else
    sLogName := sKeepNamedLogName;

  // Default extention for emergency dump
  sLogName := sLogName + '_emergency.log';

  try
    // Assign
    AssignFile( f, sLogName );
    // Rewrite
    ReWrite( f );

    // Write all entries
    for i := 0 to lstBuffer.Count - 1 do
    begin
      with lstBuffer.items[ i ]^ do
      begin
        pMyData := pData;
        pMyData := deDataEx.ReadInteger( pMyData, iType );
        case iType of
          LM_STRING:
          begin
            deDataEx.ReadString( pMyData, sMessage );
            Writeln( f, sMessage );
          end;
          LM_WARNING:
          begin
            pMyData := deDataEx.ReadInteger( pMyData, iWarnLevel );
            deDataEx.ReadString( pMyData, sMessage );
            Writeln( f, sMessage +
                     ' (WARNING level = ' + IntToStr( iWarnLevel ) + ')' );
          end;
        end;
      end;
    end;
    // Close the file
    Close( f );
  except
    // Not interested
  end;
  // Clear buffer
  lstBuffer.Clear;
end;


// Send all messages in the buffer
procedure THxLogClient.SendBuffer;
var
  i: integer;
begin
  // Send buffer to logdeamon
  for i := 0 to lstBuffer.Count - 1 do
  begin
    with lstBuffer.items[ i ]^ do
    begin
{$IFNDEF USE_UDP_CLIENT}
      SendByIpc( pData, iSize );
{$ENDIF}
{$IFDEF USE_UDP_CLIENT}
      SendByUdp( pData, iSize );
{$ENDIF}
    end;
  end;
  lstBuffer.Clear;
end;


// When you cleanup, you don't mess up...
destructor THxLogClient.Destroy;
begin
// Oops, there are still entries in the buffer
  if lstBuffer.Count > 0 then
  begin
    // If not connected write them to an emergency log file
    if not Connected then
      EmergencyDump
    else
      SendBuffer;
  end;

{$IFNDEF USE_UDP_CLIENT}
  ipcClient.Free;
{$ENDIF}

{$IFDEF USE_SERVICE_SEEKER}
  ss.Free;
{$ENDIF}

{$IFDEF USE_UDP_CLIENT}
  if Assigned( udpClient )then
  begin
    SendGoodBye; // Meldt aan de logserver dat we 'afnokken'...
    udpClient.Free;
  end;
{$ENDIF}

// Ciao buffer
  lstBuffer.Free;

  deDataEx.Free;

  inherited;
end;

{$IFDEF USE_UDP_CLIENT}
// This is the event-handler for the UDP.OnData property
// The logdaemon will only send some data back when udp connections
procedure THxLogClient.HandleUdpData( pBuf: pchar; iSize: integer;
                                      sRemoteIP, sPort: string );
var
  sCookie: string;
  iCommand: integer;
begin
// Every message from the logdaemon should start with a 'magic-cookie' this
// will prevent us from dealing with messages not ment for this application.
  pBuf := deDataEx.ReadString( pBuf, sCookie );
  if sCookie <> LOG_SERVER_COOKIE then
    exit; // Forget about it..

  bUdpConnected := True;
// A log daemon can send three kind of messages. I.e.
// 1) An acknowledge. This will be the reply after an introduction message
// 2) An 'Alive'-request
// 3) A Message that the (virtual) connection is broken.
//    This is the last message an LogDaemon will send before the logdaemon
//    disconnects. The Logdaemon will disconnect any non responding
//    applications. If an application is still alive and receives this message
//    then this class will automatically reestablish the connection.
  deDataEx.ReadInteger( pBuf, iCommand );

  case iCommand of
    UDP_ACK: HandleAcknowledge( sRemoteIP );
    UDP_REQ_ALIVE_SIGN: SendAlive;
    UDP_YOUR_LOGGED_OFF: HandleNoUdpServer;
  end;
end;
{$ENDIF}

{$IFDEF USE_UDP_CLIENT}
// This procedure will be called when the logdaemon sends a UDP-acknowledge
// message. It signals the establishment of an (virtual) connection.
// If nessecarily the udp-broadcast address will be replaced by the real
// logdaemon addrress.
procedure THxLogClient.HandleAcknowledge( sIP: string );
var
  sOldIp: string;
begin
  if bUdpConnectionEstabished then
  begin
// Keep the old address..
    sOldIp := udpClient.DestinationAddress;
// Exchange (temporaly) with this new address
    udpClient.DestinationAddress := sIP;
// Say 'thank you but we won't use our service.'.
    SendGoodBye;

// And set back the original address..
    udpClient.DestinationAddress := sOldIp;
  end
  else
  begin
    udpClient.DestinationAddress := sIP;

    if Assigned( FOnConnection )then
      FOnConnection( Self );

    bLogDaemonConnected := True;
    bUdpConnectionEstabished := True;
    bUdpConnected := True;
  end;
end;
{$ENDIF}

{$IFDEF USE_UDP_CLIENT}
procedure THxLogClient.HandleNoUdpServer;
begin
  if bLogDaemonConnected then   // Give only one warning message..
  begin
    bLogDaemonConnected := False;

    if Assigned( FOnConnectionClosed )then
      FOnConnectionClosed( Self )
    else
      Notify('Connection with Log-Daemon is BROKEN');
  end;

  {$IFDEF USE_SERVICE_SEEKER}
  ss.SeekService( LOGDAEMON_SERVICE, pcUdp );
  tmr.Enabled := False; // wait until a new service is found..
  {$ELSE}
  udpClient.DestinationAddress := GetBroadCastAddress( CURRENT_NETMASK );
  {$ENDIF}

  bUdpConnectionEstabished := False;
  bUdpConnected := False;
end;
{$ENDIF}

{$IFNDEF USE_UDP_CLIENT}
procedure THxLogClient.HandleConnection;
begin
{$IFNDEF USE_SERVICE_SEEKER}
  tmr.Enabled := False; // Just to be sure
{$ENDIF}

  SendIntroduction;

// Send buffer (if any) to logdeamon
  SendBuffer;

  if Assigned( FOnConnection )then
    FOnConnection( Self );
end;
{$ENDIF}

{$IFDEF USE_UDP_CLIENT}
procedure THxLogClient.HandleUdpTimer( Sender: TObject );
begin
  if bUdpConnected then
  begin
    bUdpConnected := False;
    exit;
  end;

  bUdpConnectionEstabished := False;
  if Assigned( FOnConnectionClosed )then
    FOnConnectionClosed( Self )
  else
    Notify('Udp-> Conection seems to broken');

  {$IFDEF USE_SERVICE_SEEKER}
  ss.SeekService( LOGDAEMON_SERVICE, pcUdp );
  {$ELSE}
  udpClient.DestinationAddress := GetBroadcastAddress( CURRENT_NETMASK );
  SendIntroduction;
  {$ENDIF}
end;
{$ENDIF}

{$IFNDEF USE_UDP_CLIENT}
procedure THxLogClient.HandleConnecionClosed;
begin
  {$IFNDEF USE_SERVICE_SEEKER}
  tmr.Enabled := True;
  {$ELSE}
  ss.SeekService( LOGDAEMON_SERVICE, pcIpc );
  {$ENDIF}

  if Assigned( FOnConnectionClosed )then
    FOnConnectionClosed( Self )
  else
    Notify('Connection with Log-Daemon is BROKEN');
end;
{$ENDIF}

{$IFNDEF USE_SERVICE_SEEKER}
  {$IFNDEF USE_UDP_CLIENT}
procedure THxLogClient.HandleIpcTimer( Sender: TObject );
begin
  tmr.Interval := 5000;
  ipcClient.ServerAddress := sDaemonAddress;
  ipcClient.Port := LOGDAEMON_PORT;

  if ipcClient.Connect then
    HandleConnection;
end;
  {$ENDIF}
{$ENDIF}

function THxLogClient.GetActive: boolean;
begin
  result := bDoLog;
end;

procedure THxLogClient.SetActive(const Value: boolean);
begin
  bDoLog := Value;
end;

procedure THxLogClient.Send( ptr: pointer; iSize: integer );
var
  pRec: PBufferRec;
begin
  if not( bDoLog )then
    exit;

  // Add to buffer if not connected yet
  if( Connected )
{$IFDEF USE_UDP_CLIENT}
  or( bForcedSend )
{$ENDIF}
  then
  begin
{$IFNDEF USE_UDP_CLIENT}
    SendByIpc( ptr, iSize );
{$ENDIF}
{$IFDEF USE_UDP_CLIENT}
    SendByUdp( ptr, iSize );
{$ENDIF}
  end
  else
  begin
    try
      // Allocate buffer record
      new( pRec );
      lstBuffer.Add( pRec );
      // Copy size
      pRec^.iSize := iSize;
      // Allocate memory for message
      pRec^.pData := AllocMem( iSize );
      // Copy message
      CopyMemory( pRec^.pData, ptr, iSize );

      // Buffer overflow => remove the oldest
      if lstBuffer.Count > iMaxBufferLines then
        lstBuffer.Delete( 0 );
    except
      // What can I do...
    end;
  end;
end;

{$IFDEF USE_UDP_CLIENT}
procedure THxLogClient.SendByUdp( ptr: pointer; iSize: integer );
begin
  udpClient.SendData( ptr, iSize );
end;
{$ENDIF}

{$IFNDEF USE_UDP_CLIENT}
procedure THxLogClient.SendByIpc( ptr: pointer; iSize: integer );
begin
  if ipcClient.Connected then
    ipcClient.SendData( ptr, iSize )
  else
    Notify( no_connection );
end;
{$ENDIF}

procedure THxLogClient.SendIntroduction;
begin
{$IFDEF USE_UDP_CLIENT}
  bForcedSend := True;
{$ENDIF}
  try
    SendStrMessage( PLM_APPLICATION_NAME, sName );
  finally
{$IFDEF USE_UDP_CLIENT}
    bForcedSend := False;
{$ENDIF}
  end;
end;

{$IFDEF USE_UDP_CLIENT}
procedure THxLogClient.SendGoodBye;
begin
  SendIntMessage( PLM_BYEBYE, 0 );
end;
{$ENDIF}

{$IFDEF USE_UDP_CLIENT}
procedure THxLogClient.SendAlive;
begin
  SendIntMessage( PLM_ALIVE_AND_KICKING, 0 );
end;
{$ENDIF}

procedure THxLogClient.LogFormated( sFormat: string; const Args: array of const );
begin
  LogStr( Format( sFormat, Args ));
end;

procedure THxLogClient.LogStr( sStr: string );
begin
  SendStrMessage( LM_STRING, sStr );
end;

procedure THxLogClient.SendStrMessage( iId: integer; s1: string );
var
  ptr: pchar;
  iSize: integer;
begin
  with deDataEx do
  begin
    StartCreatingMessage;
    AddInteger( iId );
    AddString( s1 );
    GetCreatedMessage( ptr, iSize );
  end;
  Send( ptr, iSize )
end;

procedure THxLogClient.SendIntStrMessage( iId, iVal: integer; s1: string );
var
  ptr: pchar;
  iSize: integer;
begin
  with deDataEx do
  begin
    StartCreatingMessage;
    AddInteger( iId );
    AddInteger( iVal );
    AddString( s1 );
    GetCreatedMessage( ptr, iSize );
  end;
  Send( ptr, iSize )
end;

procedure THxLogClient.SendIntMessage( iId, iVal: integer );
var
  ptr: pchar;
  iSize: integer;
begin
  with deDataEx do
  begin
    StartCreatingMessage;
    AddInteger( iId );
    AddInteger( iVal );
    GetCreatedMessage( ptr, iSize );
  end;
  Send( ptr, iSize )
end;

procedure THxLogClient.StartNewLogFile;
begin
  SendIntMessage( PLM_START_FILE, 0 );
end;

procedure THxLogClient.UseLogFileName( sName: string );
begin
  sKeepNamedLogName := sName;
  SendStrMessage( PLM_NAMED_LOG, sName );
end;

procedure THxLogClient.UseNrOfLogs( iNrOfLogs: integer );
begin
  SendIntMessage( PLM_NR_OF_FILES, iNrOfLogs );
end;

procedure THxLogClient.SetLogSwitchTime( iTimeInMin: integer );
begin
  SendIntMessage( PLM_SWITCHTIME, iTimeInMin );
end;

procedure THxLogClient.LogWarning( iWarningLevel: integer; sStr: string );
begin
  SendIntStrMessage( LM_WARNING, iWarningLevel, sStr );
end;

procedure THxLogClient.LogWarningFormated( iWarningLevel: integer;
                                         sFormat: string;
                                         const Args: array of const );
begin
  LogWarning( iWarningLevel, Format( sFormat, Args ));
end;

function THxLogClient.Connected: boolean;
begin
{$IFNDEF USE_UDP_CLIENT}
  if Assigned( ipcClient )then
    result := ipcClient.Connected
  else
    result := False;
{$ENDIF}
{$IFDEF USE_UDP_CLIENT}
  result := bLogDaemonConnected;
{$ENDIF}
end;

procedure THxLogClient.Notify( sMess: string );
begin
  if Assigned( FOnError )then
    FOnError( Self, sMess );
end;

end.



